@@ -44,7 +44,7 @@ use constant {
     IPV6MIB => 3,
 };
 
-$VERSION = '3.28';
+$VERSION = '3.53';
 
 
 
@@ -72,15 +72,18 @@ $VERSION = '3.28';
     'ip_pfx_origin'     => 'ipAddressPrefixOrigin',         # IP-MIB
     'c_pfx_origin'      => 'cIpAddressPfxOrigin',           # CISCO-IETF-IP-MIB 
 
-    'ip_addr6_pfx'      => 'ipAddressPrefix',              # IP-MIB 
+    'ip_addr6_pfx'      => 'ipAddressPrefix',               # IP-MIB
     'c_addr6_pfx'       => 'cIpAddressPrefix',              # CISCO-IETF-IP-MIB 
 
-    'ip_addr6_index'    => 'ipAddressIfIndex',              # IP-MIBw
+    'ip_addr6_pfxlen'   => 'ipAddressPrefixLength',         # IP-MIB
+    'c_addr6_pfxlen'    => 'cIpAddressPfxLength',           # CISCO-IETF-IP-MIB
+    'i6_addr_pfxlen'    => 'ipv6AddrPfxLength',             # IPV6-MIB
+
+    'ip_addr6_index'    => 'ipAddressIfIndex',              # IP-MIB
     'c_addr6_index'     => 'cIpAddressIfIndex',             # CISCO-IETF-IP-MIB 
 
     'ip_addr6_type'     => 'ipAddressType',                 # IP-MIB
     'c_addr6_type'      => 'cIpAddressType',                # CISCO-IETF-IP-MIB
-    
 );
 
 %MUNGE = (
@@ -142,6 +145,12 @@ sub ipv6_n2p_addr {
                     # so what we've collected in that variable is actually the first byte of the address.
                     $v6_packed = pack('C', $addrsize) . $v6_packed;
                 }
+                if (length($v6_packed) == 17) {
+                    # Workaround for IPV6-MIB on Windows 2012: if the address is one byte too long, the SNMP agent probably has an incorrect
+                    # implementation where a length field precedes the actual IPv6 address.
+                    # In that case, the first character should be chr(16), ie 0x10; strip it if that's the case.
+                    $v6_packed =~ s/^\x10//;
+                }
                 if (length($v6_packed) == 16) {
                     $v6addr = join(':', map { sprintf("%04x", $_) } unpack("n*", $v6_packed) );
                     $return->{$row} = $v6addr;
@@ -307,7 +316,9 @@ sub ipv6_addr_prefix {
     foreach my $row (keys %$ipv6_addr_prefix){
         if ($row =~ /^(\d+)\.[\d\.]+$/) {
             my $type = $1;
-            if ($type == 2) { # IPv6
+	    if (($type == 2) or ($type == 4)) { # IPv6
+		# Remove interface specific part from vrf interfaces
+		if ($row =~ /^((\d+\.){17}\d+)/) { $row = $1 }
 		# Remove the OID part from the value
 		my $val = $ipv6_addr_prefix->{$row};
 		if ( $val =~ /^.+?((?:\d+\.){19}\d+)$/ ){
@@ -321,9 +332,60 @@ sub ipv6_addr_prefix {
     return $return;
 }
 
+sub ipv6_addr_prefixlength {
+    my $info = shift;
+    my $return;
+    my $ipv6_addr_prefix = &_test_methods( $info, {
+        ip_addr6_pfx  => IPMIB,
+        c_addr6_pfx   => CISCO,
+    });
+    return unless defined $ipv6_addr_prefix;
+    foreach my $row (keys %$ipv6_addr_prefix) {
+        if ($row =~ /^(\d+)\.[\d\.]+$/) {
+            my $type = $1;
+            if (($type == 2) or ($type == 4)) { # IPv6
+                # Remove interface specific part from vrf interfaces
+                if ($row =~ /^((\d+\.){17}\d+)/) { $row = $1 }
+                # Remove the OID part from the value
+                my $val = $ipv6_addr_prefix->{$row};
+                if ( $val =~ /^.+?((?:\d+\.){19}(\d+))$/ ) {
+                    $val = $2;
+                    $return->{$row} = $val;
+                }
+            }
+        }
+    }
+    printf("%s: data comes from %s.\n", &_my_sub_name, $info->_method_used() ) if $info->debug();
+    return $return;
+}
+
+sub ipv6_addr {
+    my $info = shift;
+    my $return;
+    my $indexes = $info->ipv6_index();
+    foreach my $row (keys %$indexes) {
+        my @parts = split(/\./, $row);
+        my $is_valid = 0;
+        if (scalar @parts == 18) {
+            my $addrtype = shift @parts;
+            $is_valid = 1;
+        } elsif (scalar @parts == 17) {
+            $is_valid = 1;
+        }
+        my $addrsize = shift @parts; # First element now is addrsize, should be 16
+        if ($is_valid && $addrsize == 16) {
+            $return->{$row} = join(':', unpack('(H4)*', pack('C*', @parts)));
+        } else {
+            warn sprintf("%s: unable to decode table index to IPv6 address. Raw data is [%s].\n", &_my_sub_name, $row);
+        }
+    }
+    return $return;
+}
+
 sub _method_used {
     my $info = shift;
     my $return = 'none of the MIBs';
+    # FIXME ugh! a global. makes order of calls important for debug.
     if (defined $info::METHOD) {
         if ($info::METHOD eq IPMIB) {
             $return = 'IP-MIB';
@@ -343,6 +405,7 @@ sub _test_methods {
     foreach my $method (sort {$test->{$a} <=> $test->{$b}} keys %$test) {
         $return = $info->$method || {};
         if (scalar keys %$return) {
+            # FIXME ugh! a global. makes order of calls important for debug.
             $info::METHOD = $test->{$method};
             last;
         }
@@ -459,6 +522,14 @@ Maps an IPv6 prefix with its origin (manual, well-known, dhcp, etc.)
 
 Maps IPv6 addresses with their prefixes
 
+=item $info->ipv6_addr_prefixlength()
+
+Maps IPv6 addresses with their prefix length
+
+=item $info->ipv6_addr()
+
+Maps a table instance to an IPv6 address
+
 =back
 
 =head2  Internet Address Translation Table
