@@ -46,7 +46,7 @@ use SNMP::Info::Layer3;
 
 use vars qw/$VERSION %FUNCS %GLOBALS %MIBS %MUNGE/;
 
-$VERSION = '3.34';
+$VERSION = '3.53';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS,    %SNMP::Info::LLDP::MIBS,
@@ -229,7 +229,7 @@ sub index_factor {
         if ( ( $model =~ /(470)/ )
         or ( $os =~ m/(boss|bes)/ ) and ( $op_mode eq 'pure' ) );
     $index_factor = 128
-        if ( ( $model =~ /(5[56]\d\d)|VSP/ )
+        if ( ( $model =~ /(5[56]\d\d)|VSP|4950|59100/ )
         and ( $os_ver >= 6 ) );
 
     return $index_factor;
