@@ -1,7 +1,7 @@
 # SNMP::Info::Layer3::Extreme - SNMP Interface to Extreme devices
 # $Id$
 #
-# Copyright (c) 2008 Eric Miller
+# Copyright (c) 2012 Eric Miller
 #
 # Copyright (c) 2002,2003 Regents of the University of California
 # All rights reserved.
@@ -36,28 +36,36 @@ use strict;
 use Exporter;
 use SNMP::Info::Layer3;
 use SNMP::Info::MAU;
+use SNMP::Info::LLDP;
+use SNMP::Info::EDP;
 
 @SNMP::Info::Layer3::Extreme::ISA
-    = qw/SNMP::Info::Layer3 SNMP::Info::MAU Exporter/;
+    = qw/SNMP::Info::Layer3 SNMP::Info::MAU SNMP::Info::LLDP
+    SNMP::Info::EDP Exporter/;
 @SNMP::Info::Layer3::Extreme::EXPORT_OK = qw//;
 
 use vars qw/$VERSION %GLOBALS %FUNCS %MIBS %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.53';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS,
     %SNMP::Info::MAU::MIBS,
-    'EXTREME-BASE-MIB'   => 'extremeAgent',
-    'EXTREME-SYSTEM-MIB' => 'extremeSystem',
-    'EXTREME-FDB-MIB'    => 'extremeSystem',
-    'EXTREME-VLAN-MIB'   => 'extremeVlan',
-    'EXTREME-POE-MIB'    => 'extremePethSystemAdminEnable',
+    %SNMP::Info::LLDP::MIBS,
+    %SNMP::Info::EDP::MIBS,
+    'EXTREME-BASE-MIB'           => 'extremeAgent',
+    'EXTREME-SYSTEM-MIB'         => 'extremeSystem',
+    'EXTREME-FDB-MIB'            => 'extremeFdbMacFdbMacAddress',
+    'EXTREME-VLAN-MIB'           => 'extremeVlan',
+    'EXTREME-POE-MIB'            => 'extremePethSystemAdminEnable',
+    'EXTREME-STP-EXTENSIONS-MIB' => 'extremeStpDomainBridgeId',
 );
 
 %GLOBALS = (
     %SNMP::Info::Layer3::GLOBALS,
     %SNMP::Info::MAU::GLOBALS,
+    %SNMP::Info::LLDP::GLOBALS,
+    %SNMP::Info::EDP::GLOBALS,
     'serial1'        => 'extremeSystemID.0',
     'temp'           => 'extremeCurrentTemperature',
     'ps1_status_old' => 'extremePrimaryPowerOperational.0',
@@ -70,21 +78,40 @@ $VERSION = '2.06';
 %FUNCS = (
     %SNMP::Info::Layer3::FUNCS,
     %SNMP::Info::MAU::FUNCS,
+    %SNMP::Info::LLDP::FUNCS,
+    %SNMP::Info::EDP::FUNCS,
     'fan_state' => 'extremeFanOperational',
-
     # EXTREME-FDB-MIB:extremeFdbMacFdbTable
     'ex_fw_mac'    => 'extremeFdbMacFdbMacAddress',
     'ex_fw_port'   => 'extremeFdbMacFdbPortIfIndex',
     'ex_fw_status' => 'extremeFdbMacFdbStatus',
-
     # EXTREME-VLAN-MIB:extremeVlanIfTable
     'ex_vlan_descr'     => 'extremeVlanIfDescr',
     'ex_vlan_global_id' => 'extremeVlanIfGlobalIdentifier',
-
+    'ex_vlan_id'        => 'extremeVlanIfVlanId',
     # EXTREME-VLAN-MIB:extremeVlanEncapsIfTable
     'ex_vlan_encap_tag' => 'extremeVlanEncapsIfTag',
-
+    # EXTREME-VLAN-MIB:extremeVlanOpaqueTable
+    'ex_vlan_untagged'  => 'extremeVlanOpaqueUntaggedPorts',
+    'ex_vlan_tagged'    => 'extremeVlanOpaqueTaggedPorts',
+    # EXTREME-POE-MIB::extremePethPseSlotTable
+    'peth_power_watts'  => 'extremePethSlotPowerLimit',
+    # EXTREME-POE-MIB::extremePethPsePortTable
     'peth_port_power'   => 'extremePethPortMeasuredPower',
+    # EXTREME-STP-EXTENSIONS-MIB::extremeStpDomainTable
+    'stp_i_time'      => 'extremeStpDomainTimeSinceTopologyChange',
+    'stp_i_ntop'      => 'extremeStpDomainTopChanges',
+    'stp_i_root'      => 'extremeStpDomainDesignatedRoot',
+    'stp_i_root_port' => 'extremeStpDomainRootPortIfIndex',
+    'stp_i_priority'  => 'extremeStpDomainBridgePriority',
+    'ex_stp_i_mac'    => 'extremeStpDomainBridgeId',
+    # EXTREME-STP-EXTENSIONS-MIB::extremeStpPortTable
+    'stp_p_priority' => 'extremeStpPortPortPriority',
+    'stp_p_state'    => 'extremeStpPortPortState',
+    'stp_p_cost'     => 'extremeStpPortPathCost',
+    'stp_p_root'     => 'extremeStpPortDesignatedRoot',
+    'stp_p_bridge'   => 'extremeStpPortDesignatedBridge',
+    'stp_p_port'     => 'extremeStpPortDesignatedPort',
 );
 
 %MUNGE = (
@@ -92,12 +119,21 @@ $VERSION = '2.06';
     # Inherit all the built in munging
     %SNMP::Info::Layer3::MUNGE,
     %SNMP::Info::MAU::MUNGE,
-    'ex_fw_mac'      => \&SNMP::Info::munge_mac,
-    'ps1_status_old' => \&munge_true_ok,
-    'ps1_status_new' => \&munge_power_stat,
-    'ps2_status_old' => \&munge_power_stat,
-    'ps2_status_new' => \&munge_power_stat,
-    'fan_state'      => \&munge_true_ok,
+    %SNMP::Info::LLDP::MUNGE,
+    %SNMP::Info::EDP::MUNGE,
+    'ex_fw_mac'        => \&SNMP::Info::munge_mac,
+    'ps1_status_old'   => \&munge_true_ok,
+    'ps1_status_new'   => \&munge_power_stat,
+    'ps2_status_old'   => \&munge_power_stat,
+    'ps2_status_new'   => \&munge_power_stat,
+    'fan_state'        => \&munge_true_ok,
+    'ex_vlan_untagged' => \&SNMP::Info::munge_port_list,
+    'ex_vlan_tagged'   => \&SNMP::Info::munge_port_list,
+    'ex_stp_i_mac'     => \&SNMP::Info::munge_prio_mac,
+    'stp_i_root'       => \&SNMP::Info::munge_prio_mac,
+    'stp_p_root'       => \&SNMP::Info::munge_prio_mac,
+    'stp_p_bridge'     => \&SNMP::Info::munge_prio_mac,
+    'stp_p_port'       => \&SNMP::Info::munge_prio_port,
 );
 
 # Method OverRides
@@ -129,15 +165,24 @@ sub vendor {
 }
 
 sub os {
-    return 'extreme';
+    my $extreme = shift;
+
+    my $desc = $extreme->description();
+
+    if ( $desc =~ /xos/i ) {
+        return 'xos';
+    }
+
+    return 'extremeware';
 }
 
+
 sub os_ver {
     my $extreme = shift;
     my $descr   = $extreme->description();
     return unless defined $descr;
 
-    if ( $descr =~ m/Version ([\d.]*)/ ) {
+    if ( $descr =~ m/Version\s+([^ ]+)/i ) {
         return $1;
     }
 
@@ -178,7 +223,7 @@ sub i_ignore {
     my %i_ignore;
     foreach my $if ( keys %$i_description ) {
         if ( $i_description->{$if}
-            =~ /^(802.1Q Encapsulation Tag \d+|VLAN \d+|lo\d+)/i )
+            =~ /^(802.1Q Encapsulation Tag \d+|VLAN \d+|lo\d+|VirtualRouter\d+)/i )
         {
             $i_ignore{$if}++;
         }
@@ -191,6 +236,10 @@ sub i_ignore {
 # ifIndex.
 sub bp_index {
     my $extreme  = shift;
+
+    my $bindex = $extreme->SUPER::bp_index();
+    return $bindex if (keys %$bindex);
+
     my $if_index = $extreme->i_index();
 
     my %bp_index;
@@ -241,28 +290,45 @@ sub fan {
     return $ret;
 }
 
-# Newer versions of the Extreme firmware have vendor-specific tables
-# for this; those are ex_fw_*().  Older firmware versions don't have
-# these tables, so we use the BRIDGE-MIB tables.
+# For xos based VLAN functions we need to know how the ports are indexed
+# default is slot * 1000, but some older switches start at 1
+sub _slot_factor {
+    my $extreme = shift;
+    
+    my $index = $extreme->i_index();
+    
+    return 1 if (exists $index->{1} && $index->{1} == 1);
+    return 1000;
+}
+
+# Some versions of the Extreme firmware have vendor-specific tables
+# for this; those are ex_fw_*().  Some don't have these tables,
+# we use the BRIDGE-MIB tables if available then the ex_fw_*() methods.
 sub fw_mac {
     my $extreme = shift;
-    my $fw_mac  = $extreme->ex_fw_mac;
-    return $fw_mac if defined($fw_mac);
-    return $extreme->orig_fw_mac();
+    
+    my $b = $extreme->SUPER::fw_mac();
+    return $b if (keys %$b);
+
+    return $extreme->ex_fw_mac();
 }
 
 sub fw_port {
     my $extreme = shift;
-    my $fw_port = $extreme->ex_fw_port;
-    return $fw_port if defined($fw_port);
-    return $extreme->orig_fw_port();
+    
+    my $b = $extreme->SUPER::fw_port();
+    return $b if (keys %$b);
+    
+    return $extreme->ex_fw_port();
 }
 
 sub fw_status {
     my $extreme   = shift;
-    my $fw_status = $extreme->ex_fw_status;
-    return $fw_status if defined($fw_status);
-    return $extreme->orig_fw_status();
+
+    my $b = $extreme->SUPER::fw_status();
+    return $b if (keys %$b);
+
+    return $extreme->ex_fw_status();
 }
 
 # Mapping the virtual VLAN interfaces:
@@ -303,7 +369,7 @@ sub _if2tag {
         }
     }
     if ($missed) {
-        my $global_id = $extreme->ex_vlan_global_id();
+        my $global_id = $extreme->ex_vlan_id();
         foreach my $if ( keys %if2tag ) {
             $if2tag{$if} = -$global_id->{$if}
                 if ( $if2tag{$if} == -1 && defined( $global_id->{$if} ) );
@@ -312,7 +378,7 @@ sub _if2tag {
     return \%if2tag;
 }
 
-# No partial support in v_name or v_index, because the obivous partial
+# No partial support in v_name or v_index, because the obvious partial
 # is the VLAN ID and the index here is the ifIndex of
 # the VLAN interface.
 sub v_name {
@@ -322,12 +388,66 @@ sub v_name {
 
 sub v_index {
     my $extreme = shift;
-    return $extreme->_if2tag();
+    return $extreme->ex_vlan_id || $extreme->_if2tag();
 }
 
 sub i_vlan {
     my $extreme    = shift;
     my $partial    = shift;
+
+    # Some devices support Q-Bridge, if so short circuit and return it
+    my $q_bridge = $extreme->SUPER::i_vlan($partial);
+    return $q_bridge if (keys %$q_bridge);
+
+    # Next we try extremeVlanOpaqueTable
+    my $xos = $extreme->_xos_i_vlan($partial);
+    return $xos if (keys %$xos);
+    
+    # Try older ifStack method
+    my $extremeware = $extreme->_extremeware_i_vlan($partial);
+    return $extremeware if (keys %$extremeware);
+    
+    return;
+}
+
+sub _xos_i_vlan {
+    my $extreme = shift;
+    my $partial = shift;
+
+    my $index   = $extreme->i_index();
+    my $vlans   = $extreme->ex_vlan_id() || {};
+    my $slotx   = $extreme->_slot_factor() || 1000;
+    my $u_ports = $extreme->ex_vlan_untagged() || {};
+
+    my $i_vlan = {};
+    foreach my $idx ( keys %$u_ports ) {
+        next unless ( defined $u_ports->{$idx} );
+        my $portlist = $u_ports->{$idx};
+        my $ret      = [];
+
+        my ( $vlan_if, $slot ) = $idx =~ /^(\d+)\.(\d+)/;
+        my $vlan = $vlans->{$vlan_if} || '';
+
+        # Convert portlist bit array to bp_index array
+        for ( my $i = 0; $i <= $#$portlist; $i++ ) {
+            push( @{$ret}, ( $slotx * $slot + $i + 1 ) )
+                if ( @$portlist[$i] );
+        }
+
+        #Create HoA ifIndex -> VLAN array
+        foreach my $port ( @{$ret} ) {
+            my $ifindex = $index->{$port};
+            next unless ( defined($ifindex) );    # shouldn't happen
+            next if ( defined $partial and $ifindex !~ /^$partial$/ );
+            $i_vlan->{$ifindex} = $vlan;
+        }
+    }
+    return $i_vlan;
+}
+
+sub _extremeware_i_vlan {
+    my $extreme    = shift;
+    my $partial    = shift;
     my $stack      = $extreme->ifStackStatus($partial);
     my $encap_tag  = $extreme->ex_vlan_encap_tag();
     my $vlan_descr = $extreme->ex_vlan_descr();
@@ -351,6 +471,65 @@ sub i_vlan {
 }
 
 sub i_vlan_membership {
+    my $extreme = shift;
+    my $partial = shift;
+
+    # Some devices support Q-Bridge, if so short circuit and return it
+    my $q_bridge = $extreme->SUPER::i_vlan_membership($partial);
+    return $q_bridge if (ref {} eq ref $q_bridge and scalar keys %$q_bridge);
+
+    # Next we try extremeVlanOpaqueTable
+    my $xos = $extreme->_xos_i_vlan_membership($partial);
+    return $xos if (ref {} eq ref $xos and scalar keys %$xos);
+    
+    # Try older ifStack method
+    my $extremeware = $extreme->_extremeware_i_vlan_membership($partial);
+    return $extremeware if (ref {} eq ref $extremeware and scalar keys %$extremeware);
+    
+    return;
+}
+
+sub _xos_i_vlan_membership {
+    my $extreme = shift;
+    my $partial = shift;
+
+    my $index   = $extreme->i_index();
+    my $vlans   = $extreme->ex_vlan_id();
+    my $slotx   = $extreme->_slot_factor() || 1000;
+    my $u_ports = $extreme->ex_vlan_untagged() || {};
+    my $t_ports = $extreme->ex_vlan_tagged() || {};
+
+    my $i_vlan_membership = {};
+    foreach my $idx ( keys %$u_ports ) {
+        next unless ( defined $u_ports->{$idx} );
+        my $u_portlist = $u_ports->{$idx};
+        my $t_portlist = $t_ports->{$idx};
+        my $ret        = [];
+
+        my ( $vlan_if, $slot ) = $idx =~ /^(\d+)\.(\d+)/;
+        my $vlan = $vlans->{$vlan_if} || '';
+
+        foreach my $portlist ( $u_portlist, $t_portlist ) {
+
+            # Convert portlist bit array to bp_index array
+            for ( my $i = 0; $i <= $#$portlist; $i++ ) {
+                push( @{$ret}, ( $slotx * $slot + $i + 1 ) )
+                    if ( @$portlist[$i] );
+            }
+        }
+
+        #Create HoA ifIndex -> VLAN array
+        foreach my $port ( @{$ret} ) {
+            my $ifindex = $index->{$port};
+            next unless ( defined($ifindex) );    # shouldn't happen
+            next if ( defined $partial and $ifindex !~ /^$partial$/ );
+            push( @{ $i_vlan_membership->{$ifindex} }, $vlan );
+        }
+    }
+    return $i_vlan_membership;
+}
+
+sub _extremeware_i_vlan_membership {
     my $extreme    = shift;
     my $partial    = shift;
     my $stack      = $extreme->ifStackStatus($partial);
@@ -383,6 +562,78 @@ sub i_vlan_membership {
     return \%i_vlan_membership;
 }
 
+sub i_vlan_membership_untagged {
+    my $extreme = shift;
+    my $partial = shift;
+
+    # Some devices support Q-Bridge, if so short circuit and return it
+    my $q_bridge = $extreme->SUPER::i_vlan_membership_untagged($partial);
+    return $q_bridge if (ref {} eq ref $q_bridge and scalar keys %$q_bridge);
+
+    # Next we try extremeVlanOpaqueTable
+    my $xos = $extreme->_xos_i_vlan_membership_untagged($partial);
+    return $xos if (ref {} eq ref $xos and scalar keys %$xos);
+    
+    # Try older ifStack method
+    my $extremeware = $extreme->_extremeware_i_vlan_membership_untagged($partial);
+    return $extremeware if (ref {} eq ref $extremeware and scalar keys %$extremeware);
+    
+    return;
+}
+
+sub _xos_i_vlan_membership_untagged {
+    my $extreme = shift;
+    my $partial = shift;
+
+    my $index   = $extreme->i_index();
+    my $vlans   = $extreme->ex_vlan_id();
+    my $slotx   = $extreme->_slot_factor() || 1000;
+    my $u_ports = $extreme->ex_vlan_untagged() || {};
+
+    my $i_vlan_membership = {};
+    foreach my $idx ( keys %$u_ports ) {
+        next unless ( defined $u_ports->{$idx} );
+        my $u_portlist = $u_ports->{$idx};
+        my $ret        = [];
+
+        my ( $vlan_if, $slot ) = $idx =~ /^(\d+)\.(\d+)/;
+        my $vlan = $vlans->{$vlan_if} || '';
+
+        foreach my $portlist ( $u_portlist ) {
+
+            # Convert portlist bit array to bp_index array
+            for ( my $i = 0; $i <= $#$portlist; $i++ ) {
+                push( @{$ret}, ( $slotx * $slot + $i + 1 ) )
+                    if ( @$portlist[$i] );
+            }
+        }
+
+        #Create HoA ifIndex -> VLAN array
+        foreach my $port ( @{$ret} ) {
+            my $ifindex = $index->{$port};
+            next unless ( defined($ifindex) );    # shouldn't happen
+            next if ( defined $partial and $ifindex !~ /^$partial$/ );
+            push( @{ $i_vlan_membership->{$ifindex} }, $vlan );
+        }
+    }
+    return $i_vlan_membership;
+}
+
+# Assuming Cisco-like trunk behavior that native VLAN is transmitted untagged
+sub _extremeware_i_vlan_membership_untagged {
+    my $extreme  = shift;
+    my $partial = shift;
+
+    my $vlans = $extreme->_extremeware_i_vlan($partial);
+    my $i_vlan_membership = {};
+    foreach my $port (keys %$vlans) {
+        my $vlan = $vlans->{$port};
+        push( @{ $i_vlan_membership->{$port} }, $vlan );
+    }
+
+    return $i_vlan_membership;
+}
+
 # VLAN management.
 # See extreme-vlan.mib for a detailed description of
 # Extreme's use of ifStackTable and EXTREME-VLAN-MIB.
@@ -530,6 +781,97 @@ sub set_add_i_vlan_tagged {
     return $rv;
 }
 
+# LLDP uses the bridge index rather than ifIndex
+sub lldp_if {
+    my $extreme = shift;
+    my $partial = shift;
+
+    my $addr    = $extreme->lldp_rem_pid($partial) || {};
+    my $b_index = $extreme->bp_index() || {};
+    #my %r_i_descr = reverse %$i_descr;
+    
+    my %lldp_if;
+    foreach my $key ( keys %$addr ) {
+        my @aOID = split( '\.', $key );
+        my $port = $aOID[1];
+        next unless $port;
+
+        my $idx = $b_index->{$port};
+
+        $lldp_if{$key} = $idx;
+    }
+    return \%lldp_if;
+}
+
+# extremeStpDomainStpdInstance not accessible, so we need to extract from iid
+sub stp_i_id {
+    my $extreme  = shift;
+    my $partial  = shift;
+
+    my $stp_i_roots = $extreme->stp_i_root($partial);
+
+    my %stp_i_id;
+    foreach my $iid ( keys %$stp_i_roots ) {
+        $stp_i_id{$iid} = $iid;
+    }
+    return \%stp_i_id;
+}
+
+# extremeStpDomainBridgeId returns priority and mac,
+# for cross class compatibility we just need mac
+sub stp_i_mac {
+    my $extreme  = shift;
+    my $partial  = shift;
+
+    my $stp_i_bids = $extreme->ex_stp_i_mac($partial);
+
+    my %stp_i_mac;
+    foreach my $iid ( keys %$stp_i_bids ) {
+        my $mac = $stp_i_bids->{$iid};
+        next unless $mac;
+        
+        $mac =~ s/^([0-9A-F][0-9A-F]:){2}//;
+        
+        $stp_i_mac{$iid} = $mac;
+    }
+    return \%stp_i_mac;
+}
+
+# Break up the extremeStpPortEntry INDEX into Stpd Instance and IfIndex.
+sub _ex_stpport_index {
+    my $idx    = shift;
+    my ( $id, $ifindex ) = split( /\./, $idx);
+    return ($id, $ifindex);
+}
+
+# extremeStpPortPortIfIndex not-accessible, extract from iid
+sub stp_p_id {
+    my $extreme  = shift;
+    my $partial  = shift;
+
+    my $stp_port = $extreme->stp_p_root($partial);
+    my $stp_p_id  = {};
+    foreach my $idx ( keys %$stp_port ) {
+        my ( $id, $ifindex ) = _ex_stpport_index($idx);
+        $stp_p_id->{$idx} = $ifindex;
+    }
+    return $stp_p_id;
+}
+
+# extremeStpDomainStpdInstance not-accessible, extract from iid
+sub stp_p_stg_id {
+    my $extreme  = shift;
+    my $partial  = shift;
+
+    my $stp_port = $extreme->stp_p_root($partial);
+    my $stp_p_stg_id  = {};
+    foreach my $idx ( keys %$stp_port ) {
+        my ( $id, $ifindex ) = _ex_stpport_index($idx);
+        $stp_p_stg_id->{$idx} = $id;
+    }
+    return $stp_p_stg_id;
+}
+
 1;
 
 __END__
@@ -576,6 +918,10 @@ my $extreme = new SNMP::Info::Layer3::Extreme(...);
 
 =item SNMP::Info::MAU
 
+=item SNMP::Info::LLDP
+
+=item SNMP::Info::EDP
+
 =back
 
 =head2 Required MIBs
@@ -658,12 +1004,6 @@ Returns base mac
 
 =back
 
-=head2 Overrides
-
-=over
-
-=back
-
 =head2 Globals imported from SNMP::Info::Layer3
 
 See documentation in L<SNMP::Info::Layer3/"GLOBALS"> for details.
@@ -672,6 +1012,14 @@ See documentation in L<SNMP::Info::Layer3/"GLOBALS"> for details.
 
 See documentation in L<SNMP::Info::MAU/"GLOBALS"> for details.
 
+=head2 Globals imported from SNMP::Info::LLDP
+
+See documentation in L<SNMP::Info::LLDP/"GLOBALS"> for details.
+
+=head2 Globals imported from SNMP::Info::EDP
+
+See documentation in L<SNMP::Info::EDP/"GLOBALS"> for details.
+
 =head1 TABLE METHODS
 
 These are methods that return tables of information in the form of a reference
@@ -716,6 +1064,11 @@ Ignores VLAN meta interfaces and loopback
 
 (C<extremeFdbMacFdbStatus>)
 
+=item $extreme->lldp_if()
+
+Returns the mapping to the SNMP Interface Table. Extreme LLDP uses the
+bridge index rather than ifIndex.
+
 =item $extreme->i_vlan()
 
 Returns a mapping between C<ifIndex> and the VLAN.
@@ -735,6 +1088,12 @@ IDs.  These are the VLANs which are members of the egress list for the port.
     print "Port: $port VLAN: $vlan\n";
   }
 
+=item $extreme->i_vlan_membership_untagged()
+
+Returns reference to hash of arrays: key = C<ifIndex>, value = array of VLAN
+IDs.  These are the VLANs which are members of the untagged egress list for
+the port.
+
 =item $extreme->v_index()
 
 Returns VLAN IDs
@@ -756,7 +1115,106 @@ F<EXTREME-FDB-MIB> rather than F<BRIDGE-MIB>.
 =item $extreme->peth_port_power()
 
 Power supplied by PoE ports, in milliwatts
-("extremePethPortMeasuredPower")
+
+(C<extremePethPortMeasuredPower>)
+
+=item $extreme->peth_power_watts()
+
+The configured maximum amount of in-line power available to the slot.
+
+(C<extremePethSlotPowerLimit>)
+
+=back
+
+=head2 Spanning Tree Instance Globals
+
+=over
+
+=item $extreme->stp_i_mac()
+
+Returns the MAC extracted from (C<extremeStpDomainBridgeId>).
+
+=item $extreme->stp_i_id()
+
+Returns the unique identifier of the STP domain.
+
+(C<extremeStpDomainStpdInstance>)
+
+=item $extreme->stp_i_time()
+
+Returns time since last topology change detected. (100ths/second)
+
+(C<extremeStpDomainTimeSinceTopologyChange>)
+
+=item $extreme->stp_i_time()
+
+Returns time since last topology change detected. (100ths/second)
+
+(C<extremeStpDomainTimeSinceTopologyChange>)
+
+=item $extreme->stp_i_time()
+
+Returns the total number of topology changes detected.
+
+(C<extremeStpDomainTopChanges>)
+
+=item $extreme->stp_i_root()
+
+Returns root of STP.
+
+(C<extremeStpDomainDesignatedRoot>)
+
+=item $extreme->stp_i_root_port()
+
+Returns the port number of the port that offers the lowest cost path
+to the root bridge.
+
+(C<extremeStpDomainRootPortIfIndex>)
+
+=item $extreme->stp_i_priority()
+
+Returns the port number of the port that offers the lowest cost path
+to the root bridge.
+
+(C<extremeStpDomainBridgePriority>)
+
+=back
+
+=head2 Spanning Tree Protocol Port Table
+
+=over
+
+=item $extreme->stp_p_id()
+
+(C<extremeStpPortPortIfIndex>)
+
+=item $extreme->stp_p_stg_id()
+
+(C<extremeStpDomainStpdInstance>)
+
+=item $extreme->stp_p_priority()
+
+(C<extremeStpPortPortPriority>)
+
+=item $extreme->stp_p_state()
+
+(C<extremeStpPortPortState>)
+
+=item $extreme->stp_p_cost()
+
+(C<extremeStpPortPathCost>)
+
+=item $extreme->stp_p_root()
+
+(C<extremeStpPortDesignatedRoot>)
+
+=item $extreme->stp_p_bridge()
+
+(C<extremeStpPortDesignatedBridge>)
+
+=item $extreme->stp_p_port()
+
+(C<extremeStpPortDesignatedPort>)
 
 =back
 
@@ -768,6 +1226,14 @@ See documentation in L<SNMP::Info::Layer3/"TABLE METHODS"> for details.
 
 See documentation in L<SNMP::Info::MAU/"TABLE METHODS"> for details.
 
+=head2 Table Methods imported from SNMP::Info::LLDP
+
+See documentation in L<SNMP::Info::LLDP/"TABLE METHODS"> for details.
+
+=head2 Table Methods imported from SNMP::Info::EDP
+
+See documentation in L<SNMP::Info::EDP/"TABLE METHODS"> for details.
+
 =head1 SET METHODS
 
 These are methods that provide SNMP set functionality for overridden methods
@@ -818,7 +1284,6 @@ with the numeric VLAN ID and port C<ifIndex>.
   $extreme->set_remove_i_vlan_tagged('2', $if_map{'FastEthernet0/1'}) 
     or die "Couldn't add port to egress list. ",$extreme->error(1);
 
-
 =back
 
 =head1 Data Munging Callback Subroutines
