@@ -1,6 +1,6 @@
 # SNMP::Info::Layer3::H3C
 #
-# Copyright (c) 2012 Jeroen van Ingen
+# Copyright (c) 2018 Jeroen van Ingen
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -47,7 +47,7 @@ use SNMP::Info::IEEE802dot3ad 'agg_ports_lag';
 
 use vars qw/$VERSION %GLOBALS %MIBS %FUNCS %MUNGE/;
 
-$VERSION = '3.31';
+$VERSION = '3.53';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS,
@@ -71,12 +71,14 @@ $VERSION = '3.31';
 %FUNCS = (
     %SNMP::Info::Layer3::FUNCS,
     %SNMP::Info::LLDP::FUNCS,
+    %SNMP::Info::IEEE802dot3ad::FUNCS,
     i_duplex_admin => 'hh3cifEthernetDuplex',
 );
 
 %MUNGE = (
     %SNMP::Info::Layer3::MUNGE,
     %SNMP::Info::LLDP::MUNGE,
+    %SNMP::Info::IEEE802dot3ad::MUNGE,
 );
 
 sub vendor {
@@ -85,6 +87,20 @@ sub vendor {
     return $mfg->{1} || "H3C";
 }
 
+sub model {
+    my $h3c = shift;
+    
+    my $descr = $h3c->description();
+    if ($descr =~ /^.*\n(.*)\n/) {
+        return $1;
+    } elsif ($h3c->entPhysicalClass(2)->{2} =~ /^(3|chassis)$/) {
+        return $h3c->entPhysicalName(2)->{2};
+    } else {
+        my $id = $h3c->id();
+        return &SNMP::translateObj($id) || $id;
+    }
+}
+
 sub os {
     my $h3c = shift;
     my $descr   = $h3c->description();
@@ -193,6 +209,10 @@ These are methods that return scalar value from SNMP
 
 =over
 
+=item $h3c->model()
+
+Returns the model from C<sysObjectID> or C<entPhysicalClass>.
+
 =item $h3c->vendor()
 
 Returns value for C<entPhysicalMfgName.1>.
@@ -247,4 +267,8 @@ See documentation in L<SNMP::Info::Layer3> for details.
 
 See documentation in L<SNMP::Info::LLDP> for details.
 
+=head2 Table Methods imported from SNMP::Info::IEEE802dot3ad
+
+See documentation in L<SNMP::Info::IEEE802dot3ad> for details.
+
 =cut
